# AsyncMicroBenchmark
<span class="source-link">[[Source]](src/ponybench/benchmark.md#L28)</span>

Asynchronous benchmarks must provide this trait. The `apply` method defines a
single iteration in a sample. Each phase of the sample completes when the
given `AsyncBenchContinue` has its `complete` method invoked. Setup and
Teardown are defined by the `before` and `after` methods respectively. The
`before` method runs before a sample of benchmarks and `after` runs after
the all iterations in the sample have completed. If your benchmark requires
setup and/or teardown to occur beween each iteration of the benchmark, then
you must set the configuration of the benchmark to have `max_iterations = 1`.
It should be noted that a larger `sample_size` may be necessary in this
scenario for statistically significant results.


```pony
trait iso AsyncMicroBenchmark
```

## Public Functions

### name
<span class="source-link">[[Source]](src/ponybench/benchmark.md#L41)</span>


```pony
fun box name()
: String val
```

#### Returns

* [String](builtin-String) val

---

### config
<span class="source-link">[[Source]](src/ponybench/benchmark.md#L42)</span>


```pony
fun box config()
: BenchConfig val
```

#### Returns

* [BenchConfig](ponybench-BenchConfig) val

---

### overhead
<span class="source-link">[[Source]](src/ponybench/benchmark.md#L43)</span>


```pony
fun box overhead()
: AsyncMicroBenchmark iso^
```

#### Returns

* [AsyncMicroBenchmark](ponybench-AsyncMicroBenchmark) iso^

---

### before
<span class="source-link">[[Source]](src/ponybench/benchmark.md#L44)</span>


```pony
fun ref before(
  c: AsyncBenchContinue val)
: None val
```
#### Parameters

*   c: [AsyncBenchContinue](ponybench-AsyncBenchContinue) val

#### Returns

* [None](builtin-None) val

---

### apply
<span class="source-link">[[Source]](src/ponybench/benchmark.md#L45)</span>


```pony
fun ref apply(
  c: AsyncBenchContinue val)
: None val ?
```
#### Parameters

*   c: [AsyncBenchContinue](ponybench-AsyncBenchContinue) val

#### Returns

* [None](builtin-None) val ?

---

### after
<span class="source-link">[[Source]](src/ponybench/benchmark.md#L46)</span>


```pony
fun ref after(
  c: AsyncBenchContinue val)
: None val
```
#### Parameters

*   c: [AsyncBenchContinue](ponybench-AsyncBenchContinue) val

#### Returns

* [None](builtin-None) val

---

